/*
 * MATLAB MEX function for reading EXR images.
 * 
 * Only supports EXR images with 32-bit floating-point data.
 *
 */
#include <ImfInputFile.h>

#include "ImathBox.h"
#include "ImfRgba.h"
#include "ImfRgbaFile.h"
#include "ImfArray.h"
#include "ImfChannelList.h"
#include "ImfPixelType.h"
#include "Iex.h"

#include "mex.h" 

using namespace Imf;
using namespace Imath;
using namespace Iex;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	char *fileName = mxArrayToString(prhs[0]);
	Array2D<float> zPixels;
	int width, height;
	
	InputFile file (fileName);
	
	Box2i dw = file.header().dataWindow();
	width = dw.max.x - dw.min.x + 1;
	height = dw.max.y - dw.min.y + 1;
	zPixels.resizeErase (height, width);
	
	FrameBuffer frameBuffer;
	
	frameBuffer.insert ("Z", Slice (FLOAT, (char *) (&zPixels[0][0] - dw.min.x - dw.min.y * width),sizeof (zPixels[0][0]) * 1, sizeof (zPixels[0][0]) * width, 1, 1, FLT_MAX)); 
	
	file.setFrameBuffer (frameBuffer);
	file.readPixels (dw.min.y, dw.max.y);
	
	int dims[1];
	dims[0] = height;
	dims[1] = width;
	plhs[0] = mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL);
	
	double *img = mxGetPr(plhs[0]);

	int ydim = height;
	int xdim = width;
	for (int i = 0; i < ydim; ++i) 
		for (int j = 0; j < xdim; ++j) {
					int k = j*ydim + i;

					img[k]      = zPixels[i][j];
				}
			
}
 


