/*
 * MATLAB MEX function for reading EXR images.
 * 
 * Only supports EXR images with 32-bit floating-point data.
 *
 */
#include <ImfOutputFile.h>

#include "ImathBox.h"
#include "ImfRgba.h"
#include "ImfRgbaFile.h"
#include "ImfArray.h"
#include "ImfChannelList.h"
#include "ImfPixelType.h"
#include "Iex.h"

#include "mex.h" 

using namespace Imf;
using namespace Imath;
using namespace Iex;


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	char *fileName = mxArrayToString(prhs[nrhs-1]);

	int ndims = mxGetNumberOfDimensions(prhs[0]);
	int height = mxGetM(prhs[0]);
	int width = mxGetN(prhs[0]);

	int ydim = mxGetM(prhs[0]);
	int xdim = mxGetN(prhs[0]);

	double *img = mxGetPr(prhs[0]);
	
	float *zPixels = (float*) malloc(height*width*sizeof(float));
	
	for (int i = 0; i < ydim; ++i) {
		for (int j = 0; j < xdim; ++j) {
			int k = j*ydim+i;
			zPixels[i*xdim+j] = (float) img[k];
		}
	}


	Header header (width, height);
	header.channels().insert ("Z", Channel (FLOAT));
	
	OutputFile file (fileName, header); 
	FrameBuffer frameBuffer; 
	
	frameBuffer.insert ("Z", Slice (FLOAT, (char *) zPixels, sizeof (*zPixels) * 1, sizeof (*zPixels) * width)); 
	file.setFrameBuffer (frameBuffer);
	file.writePixels (height);

}

